/**
 * Copyright ProjectNya ( http://wonderfl.net/user/ProjectNya )
 * MIT License ( http://www.opensource.org/licenses/mit-license.php )
 * Downloaded from: http://wonderfl.net/c/aCJS
 */

////////////////////////////////////////////////////////////////////////////////
// Bomb (3)
////////////////////////////////////////////////////////////////////////////////

package {

    import flash.display.Sprite;
    import flash.display.Shape;
    import flash.geom.Rectangle;
    import flash.events.Event;
    import flash.events.MouseEvent;
    import flash.utils.Timer;
    import flash.events.TimerEvent;

    [SWF(backgroundColor="#EEEEEE", width="465", height="465", frameRate="30")]

    public class Main extends Sprite {
        private var bomb:Bomb;
        private static var radius:uint = 120;
        private var speed:Number = 0;
        private var bounce:Number = 0.2;
        private var acceleration:Number = 0;
        private static var gravity:uint = 4;
        private static var limit:uint = 18;
        private var manager:SoundManager;
        private var source1:String = "http://www.project-nya.jp/images/flash/bound.mp3";
        private var source2:String = "http://www.project-nya.jp/images/flash/bomb.mp3";

        public function Main() {
            init();
        }

        private function init():void {
            bomb = new Bomb(radius);
            addChild(bomb);
            bomb.x = 232;
            bomb.y = 272;
            bomb.buttonMode = true;
            bomb.addEventListener(MouseEvent.MOUSE_OVER, rollOver, false, 0, true);
            bomb.addEventListener(MouseEvent.MOUSE_OUT, rollOut, false, 0, true);
            bomb.addEventListener(MouseEvent.CLICK, click, false, 0, true);
            bomb.mouseEnabled = true;
            bomb.useHandCursor = true;
            manager = new SoundManager();
            manager.load(source1, "bound", 0.1);
            manager.load(source2, "explosion", 1);
        }
        private function rollOver(evt:MouseEvent):void {
            bomb.start();
        }
        private function rollOut(evt:MouseEvent):void {
            bomb.stop();
        }
        private function click(evt:MouseEvent):void {
            manager.play("explosion");
            bomb.mouseEnabled = false;
            bomb.useHandCursor = false;
            bomb.amplitude = radius*5;
            var timer:Timer = new Timer(500, 1);
            timer.addEventListener(TimerEvent.TIMER_COMPLETE, explode, false, 0, true);
            timer.start();
        }
        private function explode(evt:TimerEvent):void {
            removeEventListener(TimerEvent.TIMER_COMPLETE, explode);
            bomb.visible = true;
            speed = 100;
            acceleration = 200;
            addEventListener(Event.ENTER_FRAME, fall, false, 0, true);
        }
        private function fall(evt:Event):void {
            acceleration += gravity;
            bomb.amplitude += speed - acceleration;
            if (bomb.amplitude <= 0) {
                bomb.amplitude = 0;
                acceleration = 0;
                speed *= bounce;
                manager.play("bound");
            }
            if (speed < limit) {
                bomb.amplitude = 0;
                acceleration = 0;
                removeEventListener(Event.ENTER_FRAME, fall);
                bomb.mouseEnabled = true;
                bomb.useHandCursor = true;
            }
        }

    }

}


//////////////////////////////////////////////////
// Bombクラス
//////////////////////////////////////////////////

import flash.display.Sprite;
import flash.display.Shape;
import flash.events.Event;
import flash.filters.DropShadowFilter;
import flash.filters.GlowFilter;
import flash.geom.Matrix;
import flash.display.GradientType;
import flash.display.SpreadMethod;
import flash.display.InterpolationMethod;
import flash.display.BlendMode;

class Bomb extends Sprite {
    private var radius:uint;
    private var color:uint = 0x000000;
    private var bomb:Sprite;
    private var string:Shape;
    private var fire:Shape;
    private var base:Shape;
    private var shadow:Shape;
    private var light:Shape;
    private var reflection:Shape;
    private var shade:Shape;
    private static var bColor:uint = 0xFFFFFF;
    private static var sColor:uint = 0x000000;
    private static var fColor:uint = 0xFF0000;
    private var sx:Number = 0;
    private var sy:Number = 0;
    private var fx:Number = 0;
    private var fy:Number = 0;
    private var _amplitude:Number = 0;

    public function Bomb(r:uint) {
        radius = r;
        draw();
        mouseChildren = false;
    }

    private function draw():void {
        sx = radius*Math.cos(-100*Math.PI/180);
        sy = radius*Math.sin(-100*Math.PI/180);
        fx = sx-radius*0.3;
        fy = sy-radius*0.5;
        //
        shade = new Shape();
        addChild(shade);
        shade.y = radius;
        bomb = new Sprite();
        addChild(bomb);
        string = new Shape();
        bomb.addChild(string);
        fire = new Shape();
        bomb.addChild(fire);
        base = new Shape();
        bomb.addChild(base);
        shadow = new Shape();
        bomb.addChild(shadow);
        light = new Shape();
        bomb.addChild(light);
        reflection = new Shape();
        bomb.addChild(reflection);
        createBase();
        createShade();
        createShadow();
        createLight();
        createReflect();
        createString();
        createFire();
        fire.visible = false;
    }
    public function start():void {
        fire.visible = true;
        addEventListener(Event.ENTER_FRAME, update, false, 0, true);
    }
    public function stop():void {
        fire.visible = false;
        removeEventListener(Event.ENTER_FRAME, update);
    }
    private function update(evt:Event):void {
        fire.x = fx + (Math.random()-0.5)*2;
        fire.y = fy + (Math.random()-0.5)*2;
    }
    public function get amplitude():Number {
        return _amplitude;
    }
    public function set amplitude(param:Number):void {
        _amplitude = param;
        bomb.y = - _amplitude;
        var a:Number =  1 - _amplitude/(radius*5);
        if (a < 0) a = 0;
        if (a > 1) a = 1;
        shade.alpha = a;
    }
    private function createBase():void {
        base.graphics.clear();
        base.graphics.beginFill(color, 0.8);
        base.graphics.drawCircle(0, 0, radius);
        base.graphics.endFill();
    }
    private function createString():void {
        string.graphics.lineStyle(10, color, 0.8);
        string.graphics.moveTo(sx, sy);
        string.graphics.curveTo(sx-radius*0.2, sy-radius*0.1, sx-radius*0.1, sy-radius*0.25);
        string.graphics.curveTo(sx, sy-radius*0.4, fx, fy);
    }
    private function createFire():void {
        var colors:Array = [fColor, fColor, fColor];
        var alphas:Array = [1, 1, 0];
        var ratios:Array = [0, 63, 255];
        var matrix:Matrix = new Matrix();
        matrix.createGradientBox(radius*0.32, radius*0.32, 0, -radius*0.16, -radius*0.16);
        fire.graphics.beginGradientFill(GradientType.RADIAL, colors, alphas, ratios, matrix, SpreadMethod.PAD, InterpolationMethod.RGB, 0);
        fire.graphics.drawCircle(0, 0, radius*0.16);
        fire.graphics.endFill();
        fire.x = fx;
        fire.y = fy;
    }
    private function createShadow():void {
        var colors:Array = [sColor, sColor, sColor];
        var alphas:Array = [0, 0.2, 0.3];
        var ratios:Array = [0, 191, 255];
        var matrix:Matrix = new Matrix();
        matrix.createGradientBox(radius*3.2, radius*3.2, 0, -radius*2, -radius*2);
        shadow.graphics.beginGradientFill(GradientType.RADIAL, colors, alphas, ratios, matrix, SpreadMethod.PAD, InterpolationMethod.RGB, 0);
        shadow.graphics.drawCircle(0, 0, radius);
        shadow.graphics.endFill();
        shadow.blendMode = BlendMode.HARDLIGHT;
    }
    private function createLight():void {
        var colors:Array = [bColor, bColor, bColor];
        var alphas:Array = [1, 0.2, 0];
        var ratios:Array = [0, 191, 255];
        var matrix:Matrix = new Matrix();
        matrix.createGradientBox(radius*3.2, radius*3.2, 0, -radius*2, -radius*2);
        light.graphics.beginGradientFill(GradientType.RADIAL, colors, alphas, ratios, matrix, SpreadMethod.PAD, InterpolationMethod.RGB, 0);
        light.graphics.drawCircle(0, 0, radius);
        light.graphics.endFill();
        light.blendMode = BlendMode.OVERLAY;
    }
    private function createReflect():void {
        var colors:Array = [bColor, bColor];
        var alphas:Array = [0.7, 0];
        var ratios:Array = [0, 191];
        var matrix:Matrix = new Matrix();
        var w:Number = radius*1.44;
        var h:Number = radius*1.35;
        var yOffset:Number = radius*0.95;
        matrix.createGradientBox(w, h, 0.5*Math.PI, -w*0.5, -yOffset);
        reflection.graphics.beginGradientFill(GradientType.LINEAR, colors, alphas, ratios, matrix, SpreadMethod.PAD, InterpolationMethod.RGB, 0);
        reflection.graphics.drawEllipse(-w*0.5, -yOffset, w, h);
        reflection.graphics.endFill();
    }
    private function createShade():void {
        shade.graphics.beginFill(sColor, 0.6);
        shade.graphics.drawEllipse(-radius*0.75, -radius*0.1775, radius*1.5, radius*0.375);
        shade.graphics.endFill();
        var shadow:DropShadowFilter = new DropShadowFilter(0, 90, sColor, 0.5, radius*0.15, radius*0.15, 1, 3, false, false, true);
        shade.filters = [shadow];
    }

}


//////////////////////////////////////////////////
// SoundManagerクラス
//////////////////////////////////////////////////

class SoundManager {
    private var sounds:Object;
    private var levels:Object;

    public function SoundManager() {
        init();
    }

    private function init():void {
        sounds = new Object();
        levels = new Object();
    }
    public function load(file:String, id:String, level:Number):void {
        var se:SoundEffect = new SoundEffect();
        se.load(file, id);
        sounds[id] = se;
        levels[id] = level;
    }
    public function play(id:String):void {
        var se:SoundEffect = sounds[id];
        var volume:Number = levels[id];
        se.play(id, volume);
    }

}


//////////////////////////////////////////////////
// SoundEffectクラス
//////////////////////////////////////////////////

import flash.events.EventDispatcher;
import flash.events.Event;
import flash.events.ProgressEvent;
import flash.media.Sound;
import flash.media.SoundChannel;
import flash.media.SoundTransform;
import flash.events.ProgressEvent;
import flash.net.URLRequest;

class SoundEffect extends EventDispatcher {
    private var sound:Sound;
    private var channel:SoundChannel;
    private static var initialized:Boolean = false;
    private var volume:Number;
    private var looping:Boolean = false;

    public function SoundEffect() {
        if (!initialized) initialize();
    }

    private static function initialize():void {
        initialized = true;
    }
    public function init(Snd:Class):void {
        sound = Sound(new Snd());
    }
    public function load(file:String, id:String):void {
        sound = new Sound();
        sound.load(new URLRequest(file));
        sound.addEventListener(ProgressEvent.PROGRESS, progress, false, 0, true);
        sound.addEventListener(Event.COMPLETE, loaded, false, 0, true);
    }
    public function play(id:String, vol:Number, loop:Boolean = false):void {
        if (channel != null) channel.stop();
        volume = vol;
        looping = loop;
        channel = sound.play();
        var transform:SoundTransform = channel.soundTransform;
        transform.volume = volume;
        channel.soundTransform = transform;
        if (looping) {
            channel.addEventListener(Event.SOUND_COMPLETE, complete, false, 0, true);
        }
    }
    public function stop():void {
        if (channel != null) {
            channel.stop();
            channel.removeEventListener(Event.SOUND_COMPLETE, complete);
        }
    }
    private function progress(evt:ProgressEvent):void {
        dispatchEvent(evt);
    }
    private function loaded(evt:Event):void {
        dispatchEvent(evt);
    }
    private function complete(evt:Event):void {
        channel.removeEventListener(Event.SOUND_COMPLETE, complete);
        if (looping) {
            channel = sound.play(0);
            channel.addEventListener(Event.SOUND_COMPLETE, complete, false, 0, true);
            var transform:SoundTransform = channel.soundTransform;
            transform.volume = volume;
            channel.soundTransform = transform;
        }
    }

}
